////////////////////////////////////////////////////////////////////////////////////////

var video_mode = 7; // 800x600 pixels
var video_depth = 32; // 32 bit mode
//var video_screen = 1; // start the engine in full screen mode

////////////////////////////////////////////////////////////////////////////////////////

string work31_wmb = <work31.wmb>; 
string bullet_mdl = <bullet.mdl>; // always define ent_created bullet models as strings!

////////////////////////////////////////////////////////////////////////////////////////

function fire_bullets(); // creates the bullets
function move_bullets(); // moves the bullets
function remove_bullets(); // removes the bullets after they've hit something
function hurt_player();
function got_shot();
function move_enemy_bullets();

////////////////////////////////////////////////////////////////////////////////////////

define idle 1;
define attacking 2;
define dead 3;
define status skill1;
define health skill10;

////////////////////////////////////////////////////////////////////////////////////////

function main()
{
	fps_max = 80; // limit the frame rate to 80 fps
	level_load (work31_wmb);
}

action players_code
{ 
	player = my; // I'm the player
	my.invisible = on; // no need to see player's model in 1st person mode
	my.health = 100;
	my.enable_impact = on; // the player is sensitive to impact
	my.enable_entity = on; // and to other entities
	my.event = hurt_player;
	while (my.health > 0)
	{
		// move the player using the "W", "S", "A" and "D" keys; "10" = movement speed, "6" = strafing speed
		c_move (my, vector(20 * (key_w - key_s) * time, 6 * (key_a - key_d) * time, 0), nullvector, glide);
		if (mouse_left == on) // if the player presses the left mouse button
		{
			fire_bullets(); // call this function
		}
		vec_set (camera.x, player.x); // use player's x and y for the camera as well
		camera.z += 30; // place the camera 30 quants above the player on the z axis (approximate eye level)
		camera.pan -= 5 * mouse_force.x * time; // rotate the camera around by moving the mouse
		camera.tilt += 3 * mouse_force.y * time; // on its x and y axis
		player.pan = camera.pan; // the camera and the player have the same pan angle
		wait (1);
	}
	while (camera.tilt < 90)
	{
		camera.tilt += 2 * time;
		camera.roll += 1.5 * time;
		wait (1);
	}
}

function hurt_player()
{
	if (you.skill30 != 1) {return;} // didn't collide with a bullet? Then nothing should happen
	my.health -= 20; // allow the player to take up to 5 hits before dropping dead	
}

function fire_bullets()
{
	proc_kill(4); // don't allow more than 1 instance of this function to run
	while (mouse_left == on) {wait (1);} // wait until the player releases the mouse button
	ent_create (bullet_mdl, camera.x, move_bullets); // create the bullet at camera's position and attach it the "move_bullets" function
}

function move_bullets()
{
	var bullet_speed; // this var will store the speed of the bullet
	my.skill30 = 1; // I'm a bullet
	my.enable_impact = on; // the bullet is sensitive to impact
	my.enable_entity = on; // and to other entities
	my.enable_block = on; // as well as to level blocks
	my.event = remove_bullets; // when it collides with something, its event function (remove_bullets) will run
	my.pan = camera.pan; // the bullet has the same pan
	my.tilt = camera.tilt; // and tilt with the camera
	bullet_speed.x = 50 * time; // adjust the speed of the bullet here; it will move in the direction given by the camera
	bullet_speed.y = 0; // the bullet doesn't move sideways
	bullet_speed.z = 0; // or up / down on the z axis
	while (my != null) // this loop will run for as long as the bullet exists (it isn't "null")
	{
		c_move (my, bullet_speed, nullvector, ignore_you); // move the bullet ignoring its creator (the player)
		wait (1);
	}
}

function remove_bullets() // this function runs when the bullet collides with something
{
	wait (1); // wait a frame to be sure (don't trigger engine warnings)
	ent_remove (my); // and then remove the bullet
}

action my_enemy // attach this action to your enemies
{
	var idle_percentage = 0;
	var run_percentage = 0;
	var death_percentage = 0;
	my.polygon = on; // use accurate collision detection
	my.health = 100;
	my.enable_impact = on; // the enemy is sensitive to impact with player's bullets
	my.event = got_shot; // and runs this function when it is hit
	my.status = idle; // that's the same thing with my.skill1 = 1; (really!)
	while (my.status != dead) // this loop will run for as long as my.skill1 isn't equal to 3
	{
		if (my.status == idle) // hanging around?
		{
			ent_animate(my, "stand", idle_percentage, anm_cycle); // play the "stand" aka idle animation
			idle_percentage += 3 * time; // "3" controls the animation speed
			if (vec_dist (player.x, my.x) < 1000) // the player has come too close?
			{
				my.status = attacking; // then attack the player even if it hasn't fired at the enemy yet
			}
		}
		if (my.status == attacking) // shooting at the player?
		{
			vec_set(temp, player.x); 
			vec_sub(temp,my.x); 
			vec_to_angle(my.pan, temp); // turn the enemy towards the player
			if (vec_dist (player.x, my.x) > 500)
			{
				c_move (my, vector(10 * time, 0, 0), nullvector, glide);
				ent_animate(my, "run", run_percentage, anm_cycle); // play the "run" animation
				run_percentage += 6 * time; // "6" controls the animation speed
			}
			else
			{
				ent_animate(my, "alert", 100, null); // use the last frame from the "alert" animation here
			}
			if ((total_frames % 80) == 1) // fire a bullet each second
			{
				vec_for_vertex (temp, my, 8);
				// create the bullet at enemy's position and attach it the "move_enemy_bullets" function
				ent_create (bullet_mdl, temp, move_enemy_bullets); 
			}
			if (vec_dist (player.x, my.x) > 1500) // the player has moved far away from the enemy?
			{
				my.status = idle; // then switch to "idle"
			}
		}
		wait (1);
	}
	while (death_percentage < 100) // the loop runs until the "death" animation percentage reaches 100%	
	{
		ent_animate(my, "deatha", death_percentage, null); // play the "die" animation only once
		death_percentage += 3 * time; // "3" controls the animation speed
		wait (1);
	}
	my.passable = on; // allow the player to pass through the corpse now
}

function got_shot()
{
	if (you.skill30 != 1) {return;} // didn't collide with a bullet? Then nothing should happen
	my.health -= 35;
	if (my.health <= 0) // dead?
	{
		my.status = dead; // stop the loop from "action my_enemy"
		my.event = null; // the enemy is dead, so it should stop reacting to other bullets from now on
		return; // end this function here
	}
	else // got shot but not (yet) dead?
	{
		my.status = attacking; // same thing with my.skill1 = 2
	}
}

function move_enemy_bullets()
{
	var bullet_speed; // this var will store the speed of the bullet
	my.skill30 = 1; // I'm a bullet
	my.enable_impact = on; // the bullet is sensitive to impact
	my.enable_entity = on; // and to other entities
	my.enable_block = on; // as well as to level blocks
	my.event = remove_bullets; // when it collides with something, its event function (remove_bullets) will run
	my.pan = you.pan; // the bullet has the same pan
	my.tilt = you.tilt; // and tilt with the enemy
	bullet_speed.x = 50 * time; // adjust the speed of the bullet here
	bullet_speed.y = 0; // the bullet doesn't move sideways
	bullet_speed.z = 0; // or up / down on the z axis
	// the loop will run for as long as the bullet exists (it isn't "null")
	while (my != null) 
	{
		// move the bullet ignoring its creator (the enemy)
		c_move (my, bullet_speed, nullvector, ignore_you); 
		wait (1);
	}
}